/******************************************************************************
*                                                                             *
*   t i c o n s t                                                             *
*                                                                             *
*   by Stefan Heule                                                           *
*   http://stefanheule.com                                                    *
*                                                                             *
*   Source Code                                                               *
*                                                                             *
*******************************************************************************

ticonst - replacement for the standard units menu
Copyright (C) 2009-2010 Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef HEADER_H
#define HEADER_H



// ----------------------------------------------------------------- //
// --- defines
// ----------------------------------------------------------------- //

// smaller code
#define OPTIMIZE_ROM_CALLS

// min ams version
#define MIN_AMS 200

// if you change the backup struct, change this magic, too!
// ticons01  ->  ticonst 1.1.0
#define OPTIONS_FILE_MAGIC        "ticons01"

// name of the backup file
#define OPTIONS_FILE_NAME "main\\ticoncfg"
#define OPTIONS_FILE_EXT "lst"

typedef struct _custom_font
{
	char *font4x6;
	char *font6x8;
	unsigned short *font4x6width;
} CUST_FONT;
extern CUST_FONT myfont;

typedef struct 
{
	const char* name;
	const char* desc;
} _unit;

typedef struct 
{
	const char* name;
	const _unit* lst;
} _group;


typedef struct _options
{
	short lm_up; // index of first group that is displayed
	short lm_mid; // cursor pos (group)
	short rm_up; // index of first item that is displayed
	short rm_mid; // cursor pos (item)
	short save_pos;  // save current postition?
	short cur ; // 0 = left menu, 1 = right menu
	short mode; // 0 = name, 1 = value
} OPTIONS;


const unsigned char checkmark_sprt[] = {
	0b00000000,
	0b00000001,
	0b00000010,
	0b00000100,
	0b10001000,
	0b01010000,
	0b00100000,
	0b00000000,
};



// ----------------------------------------------------------------- //
// --- header files
// ----------------------------------------------------------------- //
#include <tigcclib.h>
#include "messages.h"
#include "extgraph.h"


// ----------------------------------------------------------------- //
// --- extern vars
// ----------------------------------------------------------------- //
extern OPTIONS pref;    // the preferences
extern void* screen;    // backbuffer + screen backup


// ----------------------------------------------------------------- //
// --- prototypes
// ----------------------------------------------------------------- //
// main.c
void _main(void);

// ui.c
void switchBackbuffer(void);
void DrawDialog(short x0, short y0, short x1, short y1, const char* title, char help);
void DrawRect(short x0, short y0, short x1, short y1, short fill, short color);
short Strlen(const char* str);
void FastDrawString4x6(short x, short y, const char *str, short color);
void FastDrawString6x8(short x, short y, const char *str, short color);
short InitializeFastString(void);
void FreeFonts(void);

// util.c
short getKey(short allow_repeat);
short isKeyWaiting(void);

// fileio.c
short loadBackup(void);
unsigned char checksum(void* data, unsigned short bytes);
short saveBackup(void);
void deleteBackup(void);

#endif // #ifndef HEADER_H
















