/******************************************************************************
*                                                                             *
*   t i c o n s t                                                             *
*                                                                             *
*   by Stefan Heule                                                           *
*   http://stefanheule.com                                                    *
*                                                                             *
*   Source Code                                                               *
*                                                                             *
*******************************************************************************

ticonst - replacement for the standard units menu
Copyright (C) 2009-2010 Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/



// ----------------------------------------------------------------- //
// --- header files
// ----------------------------------------------------------------- //
#include "header.h"



// ----------------------------------------------------------------- //
// copies the content of the backbuffer <screen> onto the real screen
// ----------------------------------------------------------------- //
void switchBackbuffer(void)
{
	memcpy(LCD_MEM, screen, LCD_SIZE);
}

// ----------------------------------------------------------------- //
// draws a dialog with the coordinates <x0/y0> and <x1/y1> and the
// title <title>
// ----------------------------------------------------------------- //
void DrawDialog(short x0, short y0, short x1, short y1, const char* title, char help)
{
	// background (white)
	DrawRect(x0+1, y0+7, x1-1, y1-1, 1, 0);
	
	// titelbar
	DrawRect(x0, y0+2, x1, y0+6, 1, 1);
	DrawLine(x0+2,y0 ,x1-2, y0, 1);
	DrawLine(x0+1, y0+1, x1-1, y0+1, 1);
	
	// border arround
	DrawLine(x0+2, y1, x1-2, y1, 1);
	DrawLine(x0, y0+7,x0 ,y1-2, 1);
	DrawLine(x1, y0+7,x1 ,y1-2, 1);
	DrawPix(x0+1, y1-1, 1);
	DrawPix(x1-1, y1-1, 1);
	
	// border inside the window
	DrawRect(x0+2, y0+8, x1-2, y1-2, 0, 1);
	
	// title
	FastDrawString4x6(x0+5, y0+1, title, A_XOR);
	if (help) FastDrawString4x6(x1-5-Strlen(MSG_HELP_TITLE), y0+1, MSG_HELP_TITLE, A_XOR);
};

// ----------------------------------------------------------------- //
// draws a rect with the coordinates <x0/y0> and <x1/y1>
// if <fill> is 0, the rect will be outlined, else filled
// <color> = 1: normal
// <color> = 2: double border
// <color> = 3: xor
// else:        erase
// ----------------------------------------------------------------- //
void DrawRect(short x0, short y0, short x1, short y1, short fill, short color)
{
	short att;
	if (color == 1)      att = A_NORMAL;
	else if (color == 2) att = A_NORMAL | B_DOUBLE;
	else if (color == 3) att = A_XOR;
	else                 att = A_REVERSE;
	
	if (fill) 
		ScrRectFill(&(SCR_RECT){{x0, y0, x1, y1}}, &(SCR_RECT){{0, 0, 239, 127}}, att);
	else
		DrawClipRect (&(WIN_RECT){x0, y0, x1, y1}, &(SCR_RECT){{0, 0, 239, 127}}, att);
	
	/*if (fill) {
		if (color == 1)
			FastFilledRect_Draw_R(screen, x0, y0, x1, y1);
		else if (color == 3)
			FastFilledRect_Invert_R(screen, x0, y0, x1, y1);
		else
			FastFilledRect_Erase_R(screen, x0, y0, x1, y1);
	}
	else {
		if (color == 1)
			FastOutlineRect_R(screen, x0, y0, x1, y1, A_NORMAL);
		else if (color == 3)
			FastOutlineRect_R(screen, x0, y0, x1, y1, A_XOR);
		else
			FastOutlineRect_R(screen, x0, y0, x1, y1, A_REVERSE);
	}*/
	
};


// get the width of a 4x6 non-fixed-width string
short Strlen(const char* str)
{
	unsigned short width = 0;
	
	while (*str)
	{
		width += myfont.font4x6width[(unsigned char)*str++];
	}
	
	return width;
}

// fast string drawing (F_4x6 font)
void FastDrawString4x6(short x, short y, const char *str, short mode)
{
	short xpos = x;
	
	while ((*str) && (xpos + 4 < 240) && (y + 6 < 128))
	{
		//GraySingleSprite8_COLOR_R(xpos, y, 5, &myfont.font4x6[((unsigned char)*str)*5], color, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(DARK_PLANE));
		if (mode == A_XOR)
			Sprite8_XOR_R(xpos, y, 5, &myfont.font4x6[((unsigned char)*str)*5], screen);
		else if (mode == A_NORMAL)
			Sprite8_AND_R(xpos, y, 5, &myfont.font4x6[((unsigned char)*str)*5], screen);
		else if (mode == A_REVERSE)
			Sprite8_AND_R(xpos, y, 5, &myfont.font4x6[((unsigned char)*str)*5], screen);
		
		xpos += myfont.font4x6width[((unsigned char)*str)];
		str++;
	}
}

// fast string drawing (F_6x8 font)
void FastDrawString6x8(short x, short y, const char *str, short mode)
{
	short xpos = x;
	
	while ((*str) && (xpos + 6 < 240) && (y + 8 < 128))
	{
		//GraySingleSprite8_COLOR_R(xpos, y, 8, &myfont.font6x8[((unsigned char)*str)*8], color, GrayGetPlane(LIGHT_PLANE), GrayGetPlane(DARK_PLANE)); 
		
		if (mode == A_XOR)
			Sprite8_XOR_R(xpos, y, 8, &myfont.font6x8[((unsigned char)*str)*8], screen);
		else if (mode == A_NORMAL)
			Sprite8_AND_R(xpos, y, 8, &myfont.font6x8[((unsigned char)*str)*8], screen);
		else if (mode == A_REVERSE)
			Sprite8_AND_R(xpos, y, 8, &myfont.font6x8[((unsigned char)*str)*8], screen);
		
		xpos += 6;
		str++;
	}
}

short InitializeFastString(void)
{
	unsigned short i;
	
	// allocate some space for the font4x6/font4x6width-array
	if ((myfont.font4x6 = malloc(256*5+256*sizeof(short))) == NULL)
		return 0;
   
	// set the address of font4x6width
	myfont.font4x6width = (short*)(myfont.font4x6+256*5);
	
	FontSetSys(F_4x6);
	memset(myfont.font4x6, 0, 256*5);
	PortSet(myfont.font4x6, 7, 5*256-1);
	
	// now draw all characters into the buffer and get the width of every character
	for (i = 0; i < 256; i++)
	{
		myfont.font4x6width[i] = FontCharWidth(i);
		DrawChar(0, i*5, i, A_REPLACE);
	}
	
	// allocate some space for the font6x8-array
	if ((myfont.font6x8 = malloc(256*8)) == NULL)
	{
		free(myfont.font4x6);
		return 0;
	}
	
	FontSetSys(F_6x8);
	
	// clear the array
	memset(myfont.font6x8, 0, 256*8);
	PortSet(myfont.font6x8, 7, 8*256-1);
	
	// now draw all characters into the buffer
	for (i = 0; i < 256; i++)
	{
		DrawChar(0, i*8, i,A_REPLACE);
	}
	
	PortRestore();
	
	return 1;
}

// frees all allocated memory blocks
void FreeFonts(void)
{
	free(myfont.font4x6);
	free(myfont.font6x8);
}


















